package com.xinelu.manage.mapper.servicepackagecontent;

import com.xinelu.manage.domain.servicepackagecontent.ServicePackageContent;
import com.xinelu.manage.vo.servicepackagecontent.ServicePackageContentVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 服务包内容Mapper接口
 *
 * @author xinelu
 * @date 2024-03-04
 */
public interface ServicePackageContentMapper {
    /**
     * 查询服务包内容
     *
     * @param id 服务包内容主键
     * @return 服务包内容
     */
    public ServicePackageContent selectServicePackageContentById(Long id);

    /**
     * 查询服务包内容列表
     *
     * @param servicePackageContent 服务包内容
     * @return 服务包内容集合
     */
    public List<ServicePackageContent> selectServicePackageContentList(ServicePackageContent servicePackageContent);

    /**
     * 新增服务包内容
     *
     * @param servicePackageContent 服务包内容
     * @return 结果
     */
    public int insertServicePackageContent(ServicePackageContent servicePackageContent);

    /**
     * 修改服务包内容
     *
     * @param servicePackageContent 服务包内容
     * @return 结果
     */
    public int updateServicePackageContent(ServicePackageContent servicePackageContent);

    /**
     * 删除服务包内容
     *
     * @param id 服务包内容主键
     * @return 结果
     */
    public int deleteServicePackageContentById(Long id);

    /**
     * 批量删除服务包内容
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteServicePackageContentByIds(Long[] ids);

    /**
     * 根据服务包id删除服务包内容
     */
    int deleteServicePackageContentByPackageIds(Long[] ids);

    /**
     * 根据服务包id和服务方式名称查询服务方式是否存在
     */
    Long countByPackageIdAndServiceWayName(@Param("id") Long id, @Param("serviceWayName") String serviceWayName);

    /**
     * 根据服务方式id和服务内容查询服务内容是否存在
     */
    Long countByServiceWayIdAndServiceContent(@Param("serviceWayId") Long serviceWayId, @Param("serviceContent") String serviceContent);

    /**
     * 根据服务内容id和服务频次查询服务频次是否存在
     */
    int countByServiceContentIdAndServiceFrequency(@Param("serviceContentId") Long serviceContentId, @Param("vo") ServicePackageContentVO vo);

    /**
     * 根据服务包id删除服务包内容
     */
    int deleteServicePackageContentByPackageId(Long id);
}
